/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

                Copyright (c) 2003-2019, Cauldron Development LLC
                   Copyright (c) 2003-2017, Stanford University
                               All rights reserved.

         The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
       as published by the Free Software Foundation, either version 2.1 of
               the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                 Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
                 License along with the C! library.  If not, see
                         <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
           You may request written permission by emailing the authors.

                  For information regarding this software email:
                                 Joseph Coffland
                          joseph@cauldrondevelopment.com

\******************************************************************************/

#pragma once

#include <cbang/SmartPointer.h>

#include <vector>
#include <ostream>


namespace cb {
  namespace JSON {class Sink;}

  namespace js {
    class Value {
    public:
      virtual ~Value() {}

      virtual SmartPointer<Value> makePersistent() const = 0;

      // Read interface
      virtual bool isArray() const {return false;}
      virtual bool isBoolean() const {return false;}
      virtual bool isFunction() const {return false;}
      virtual bool isNull() const {return false;}
      virtual bool isNumber() const {return false;}
      virtual bool isObject() const {return false;}
      virtual bool isString() const {return false;}
      virtual bool isUndefined() const {return false;}

      virtual bool toBoolean() const = 0;
      virtual int toInteger() const = 0;
      virtual double toNumber() const = 0;
      virtual std::string toString() const = 0;

      virtual unsigned length() const = 0;
      virtual SmartPointer<Value> get(int i) const = 0;

      virtual bool has(const std::string &key) const = 0;
      virtual SmartPointer<Value> get(const std::string &key) const = 0;
      virtual SmartPointer<Value> getOwnPropertyNames() const = 0;

      // Array accessors
      bool getBoolean(int i) const {return get(i)->toBoolean();}
      int getInteger(int i) const {return get(i)->toInteger();}
      double getNumber(int i) const {return get(i)->toNumber();}
      std::string getString(int i) const {return get(i)->toString();}
      SmartPointer<Value> operator[](int i) const {return get(i);}

      // Object accessors
      bool getBoolean(const std::string &key) const
      {return get(key)->toBoolean();}
      int getInteger(const std::string &key) const
      {return get(key)->toInteger();}
      double getNumber(const std::string &key) const
      {return get(key)->toNumber();}
      std::string getString(const std::string &key) const
      {return get(key)->toString();}
      SmartPointer<Value> operator[](const std::string &key) const
      {return get(key);}

      // Function accessors
      virtual SmartPointer<Value>
      call(const std::vector<SmartPointer<Value> > &args) const = 0;

      // Write interface
      virtual void set(int i, const Value &value) = 0;
      virtual void set(const std::string &key, const Value &value) = 0;
      virtual void set(int i, const SmartPointer<Value> &value)
      {set(i, *value);}
      virtual void set(const std::string &key,
                       const SmartPointer<Value> &value) {set(key, *value);}

      void copyProperties(const Value &value);
      void write(JSON::Sink &sink) const;
      void write(std::ostream &stream) const;
    };


    inline static
    std::ostream &operator<<(std::ostream &stream, const Value &value) {
      value.write(stream);
      return stream;
    }
  }
}
